@@ -2,7 +2,7 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-// +build darwin dragonfly freebsd linux nacl netbsd openbsd solaris
+// +build aix darwin dragonfly freebsd hurd js,wasm linux netbsd openbsd solaris
 
 package time
 
@@ -16,32 +16,6 @@ func interrupt() {
 	syscall.Kill(syscall.Getpid(), syscall.SIGCHLD)
 }
 
-// readFile reads and returns the content of the named file.
-// It is a trivial implementation of ioutil.ReadFile, reimplemented
-// here to avoid depending on io/ioutil or os.
-func readFile(name string) ([]byte, error) {
-	f, err := syscall.Open(name, syscall.O_RDONLY, 0)
-	if err != nil {
-		return nil, err
-	}
-	defer syscall.Close(f)
-	var (
-		buf [4096]byte
-		ret []byte
-		n   int
-	)
-	for {
-		n, err = syscall.Read(f, buf[:])
-		if n > 0 {
-			ret = append(ret, buf[:n]...)
-		}
-		if n == 0 || err != nil {
-			break
-		}
-	}
-	return ret, err
-}
-
 func open(name string) (uintptr, error) {
 	fd, err := syscall.Open(name, syscall.O_RDONLY, 0)
 	if err != nil {
@@ -50,6 +24,10 @@ func open(name string) (uintptr, error) {
 	return uintptr(fd), nil
 }
 
+func read(fd uintptr, buf []byte) (int, error) {
+	return syscall.Read(int(fd), buf)
+}
+
 func closefd(fd uintptr) {
 	syscall.Close(int(fd))
 }
@@ -74,5 +52,3 @@ func preadn(fd uintptr, buf []byte, off int) error {
 	}
 	return nil
 }
-
-func isNotExist(err error) bool { return err == syscall.ENOENT }
