@@ -1,8 +1,7 @@
 // { dg-require-namedlocale "en_US.ISO8859-1" }
+// { dg-require-namedlocale "fr_FR.ISO8859-15" }
 
-// 2004-04-16  Petur Runolfsson  <peturr02@ru.is>
-
-// Copyright (C) 2004-2017 Free Software Foundation, Inc.
+// Copyright (C) 2003-2021 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -25,39 +24,28 @@
 #include <locale>
 #include <testsuite_hooks.h>
 
-class Buf : public std::filebuf
+class Cvt : public std::codecvt<char, char, std::mbstate_t>
 {
 protected:
-  virtual int_type
-  overflow(int_type c = traits_type::eof())
-  {
-    return traits_type::eq_int_type(c, traits_type::eof()) ?
-      traits_type::eof() : std::filebuf::overflow(c);
-  }
+  int do_encoding() const throw()
+  { return -1; }
+
+  bool do_always_noconv() const throw()
+  { return false; }
 };
 
-// libstdc++/14975
+// libstdc++/13171
 void test01()
 {
   using namespace std;
 
-  Buf fb;
-  locale loc_us = locale(ISO_8859(1,en_US));
-  fb.pubimbue(loc_us);
-  fb.open("tmp_14975-1", ios_base::out);
-  
-  try
-    {
-      fb.sputc('a');
-      fb.sputc('b');
-      fb.pubimbue(locale::classic());
-      fb.sputc('c');
-      fb.pubsync();
-      fb.close();
-    }
-  catch (std::exception&)
-    {
-    }
+  filebuf fb;
+  fb.pubimbue(locale(locale(ISO_8859(1,en_US)), new Cvt));
+  fb.open("tmp_13171-4", ios_base::out);
+  fb.pubimbue(locale(ISO_8859(15,fr_FR)));
+  fb.sputc('N');
+  fb.pubsync();
+  fb.close();
 }
 
 int main()
